@@ -1,30 +1,29 @@
 from time import sleep as slp
 from sys import exit
 from json import dumps, load
-from string import hexdigits
 from pathlib import Path
 from threading import Thread
 from contextlib import suppress
-from random import sample, choice
-from schedule import every, run_pending
-
-try:
-    from amino import Client, SubClient, ACM
-except ModuleNotFoundError:
-    from amino2bot import Client, SubClient, ACM
+from unicodedata import normalize
+from string import punctuation
+from random import choice
+from amino import Client, SubClient, ACM
+from uuid import uuid4
 # this is the Slimakoi's API with some of my patches
 
 # API made by ThePhoenix78
 # Big optimisation thanks to SempreLEGIT#1378 ♥
 
+
 path_utilities = "utilities"
 path_amino = f'{path_utilities}/amino_list'
 path_client = "client.txt"
 NoneType = type(None)
 
 
-for i in (path_utilities, path_amino):
-    Path(i).mkdir(exist_ok=True)
+with suppress(Exception):
+    for i in (path_utilities, path_amino):
+        Path(i).mkdir(exist_ok=True)
 
 
 def print_exception(exc):
@@ -37,76 +36,155 @@ class Command:
         self.conditions = {}
 
     def execute(self, commande, data, type: str = "command"):
-        if commande in self.conditions.keys():
-            if self.conditions[commande](data):
+        if commande in self.conditions[type].keys():
+            if self.conditions[type][commande](data):
                 return self.commands[type][commande](data)
             else:
                 return
         return self.commands[type][commande](data)
 
-    def add_categorie(self, name):
-        if name not in self.commands.keys():
-            self.commands[name] = {}
+    def categorie_exist(self, type: str):
+        return type in self.commands.keys()
+
+    def add_categorie(self, type):
+        if type not in self.commands.keys():
+            self.commands[type] = {}
+
+    def add_condition(self, type):
+        if type not in self.conditions.keys():
+            self.conditions[type] = {}
 
     def commands_list(self):
-        return [command.keys() for command in self.commands.keys()]
+        return [command.keys() for command in self.commands["command"].keys()]
 
-    def command(self, command_name, condition=None):
-        name = "command"
-        self.add_categorie(name)
+    def answer_list(self):
+        return [command.keys() for command in self.commands["answser"].keys()]
+
+    def command(self, name=None, condition=None):
+        type = "command"
+        self.add_categorie(type)
+        self.add_condition(type)
+        if isinstance(name, str):
+            name = [name]
+        elif not name:
+            name = []
+
+        def add_command(command_funct):
+            name.append(command_funct.__name__)
+            if callable(condition):
+                for command in name:
+                    self.conditions[type][command] = condition
+
+            for command in name:
+                self.commands[type][command.lower()] = command_funct
+            return command_funct
+
+        return add_command
+
+    def answer(self, name, condition=None):
+        type = "answer"
+        self.add_categorie(type)
+        self.add_condition(type)
+
+        if isinstance(name, str):
+            name = [name]
+        elif not name:
+            name = []
+
+        def add_command(command_funct):
+            # name.append(command_funct.__name__)
+            if callable(condition):
+                for command in name:
+                    self.conditions[type][command] = condition
+
+            for command in name:
+                self.commands[type][command.lower()] = command_funct
+            return command_funct
+
+        return add_command
+
+    def on_member_join_chat(self, condition=None):
+        type = "on_member_join_chat"
+        self.add_categorie(type)
+        self.add_condition(type)
         if callable(condition):
-            self.conditions[command_name] = condition
+            self.conditions[type][type] = condition
 
         def add_command(command_funct):
-            self.commands[name][command_name.lower()] = command_funct
+            self.commands[type][type] = command_funct
             return command_funct
         return add_command
 
-    def answer(self, command_name, condition=None):
-        name = "answer"
-        self.add_categorie(name)
+    def on_member_leave_chat(self, condition=None):
+        type = "on_member_leave_chat"
+        self.add_categorie(type)
+        self.add_condition(type)
         if callable(condition):
-            self.conditions[command_name] = condition
+            self.conditions[type][type] = condition
 
         def add_command(command_funct):
-            self.commands[name][command_name.lower()] = command_funct
+            self.commands[type][type] = command_funct
             return command_funct
         return add_command
 
-    def on_member_join_chat(self, chatId: list = None):
-        name = "on_member_join_chat"
+    def on_message(self, condition=None):
+        type = "on_message"
+        self.add_categorie(type)
+        self.add_condition(type)
+        if callable(condition):
+            self.conditions[type][type] = condition
 
         def add_command(command_funct):
-            if isinstance(self.commands[name], list):
-                self.commands[name].append(command_funct)
-            elif isinstance(self.commands[name], dict):
-                for chat in chatId:
-                    self.commands[name][chat] = command_funct
+            self.commands[type][type] = command_funct
             return command_funct
-
-        if name not in self.commands.keys() and chatId:
-            self.commands[name] = {}
-        elif not chatId:
-            self.commands[name] = []
-
         return add_command
 
-    def on_member_leave_chat(self, chatId: list = None):
-        name = "on_member_leave_chat"
+    def on_other(self, condition=None):
+        type = "on_other"
+        self.add_categorie(type)
+        self.add_condition(type)
+        if callable(condition):
+            self.conditions[type][type] = condition
 
         def add_command(command_funct):
-            if isinstance(self.commands[name], list):
-                self.commands[name].append(command_funct)
-            elif isinstance(self.commands[name], dict):
-                for chat in chatId:
-                    self.commands[name][chat] = command_funct
+            self.commands[type][type] = command_funct
             return command_funct
+        return add_command
 
-        if name not in self.commands.keys() and chatId:
-            self.commands[name] = {}
-        elif not chatId:
-            self.commands[name] = []
+    def on_delete(self, condition=None):
+        type = "on_delete"
+        self.add_categorie(type)
+        self.add_condition(type)
+        if callable(condition):
+            self.conditions[type][type] = condition
 
+        def add_command(command_funct):
+            self.commands[type][type] = command_funct
+            return command_funct
+        return add_command
+
+    def on_remove(self, condition=None):
+        type = "on_remove"
+        self.add_categorie(type)
+        self.add_condition(type)
+        if callable(condition):
+            self.conditions[type][type] = condition
+
+        def add_command(command_funct):
+            self.commands[type][type] = command_funct
+            return command_funct
+        return add_command
+
+    def on_all(self, condition=None):
+        type = "on_all"
+        self.add_categorie(type)
+        self.add_condition(type)
+        if callable(condition):
+            self.conditions[type][type] = condition
+
+        def add_command(command_funct):
+            self.commands[type][type] = command_funct
+            return command_funct
         return add_command
 
 
@@ -130,8 +208,23 @@ class TimeOut:
         return True
 
 
+class BannedWords:
+    def filtre_message(self, message, code):
+        para = normalize('NFD', message).encode(code, 'ignore').decode("utf8").strip().lower()
+        para = para.translate(str.maketrans("", "", punctuation))
+        return para
+
+    def check_banned_words(self, args):
+        for word in ("ascii", "utf8"):
+            with suppress(Exception):
+                para = self.filtre_message(args.message, word).split()
+                if para != [""]:
+                    with suppress(Exception):
+                        [args.subClient.delete_message(args.chatId, args.messageId, reason="Banned word", asStaff=True) for elem in para if elem in args.subClient.banned_words]
+
+
 class Parameters:
-    __slots__ = ("subClient", "chatId", "authorId", "author", "message", "messageId")
+    __slots__ = ("subClient", "chatId", "authorId", "author", "message", "messageId", "info")
 
     def __init__(self, data, subClient):
         self.subClient = subClient
@@ -140,12 +233,13 @@ class Parameters:
         self.author = data.message.author.nickname
         self.message = data.message.content
         self.messageId = data.message.messageId
+        self.info = data
 
 
-class BotAmino(Command, Client, TimeOut):
-    def __init__(self, email: str = None, password: str = None, sid: str = None):
+class BotAmino(Command, Client, TimeOut, BannedWords):
+    def __init__(self, email: str = None, password: str = None, sid: str = None,  proxies: dict = None):
         Command.__init__(self)
-        Client.__init__(self)
+        Client.__init__(self, proxies=proxies)
 
         if email and password:
             self.login(email=email, password=password)
@@ -168,9 +262,12 @@ class BotAmino(Command, Client, TimeOut):
         self.len_community = 0
         self.perms_list = []
         self.prefix = "!"
+        self.activity = False
         self.wait = 0
         self.bio = None
         self.self_callable = False
+        self.no_command_message = ""
+        self.spam_message = "You are spamming, be careful"
 
     def tradlist(self, sub):
         sublist = []
@@ -198,7 +295,7 @@ class BotAmino(Command, Client, TimeOut):
         return self.get_wallet_info().totalCoins
 
     def generate_transaction_id(self):
-        return f"{''.join(sample([lst for lst in hexdigits[:-6]], 8))}-{''.join(sample([lst for lst in hexdigits[:-6]], 4))}-{''.join(sample([lst for lst in hexdigits[:-6]], 4))}-{''.join(sample([lst for lst in hexdigits[:-6]], 4))}-{''.join(sample([lst for lst in hexdigits[:-6]], 12))}"
+        return str(uuid4())
 
     def check(self, args, *can, id_=None):
         id_ = id_ if id_ else args.authorId
@@ -209,33 +306,62 @@ class BotAmino(Command, Client, TimeOut):
             if foo[i](id_):
                 return True
 
-    def add_community(self, comId):
-        self.communaute[comId] = Bot(self, comId, self.prefix, self.bio)
+    def threadLaunch(self, commu, passive):
+        self.communaute[commu] = Bot(self, commu, self.prefix, self.bio, self.activity)
+        slp(30)
+        if passive:
+            self.communaute[commu].passive()
 
-    def run(self, comId):
-        self.communaute[comId].run()
-
-    def threadLaunch(self, commu):
-        self.add_community(commu)
-        self.run(commu)
-
-    def launch(self):
+    def launch(self, passive: bool = False):
         amino_list = self.sub_clients()
         self.len_community = len(amino_list.comId)
-        [Thread(target=self.threadLaunch, args=[commu]).start() for commu in amino_list.comId]
+        [Thread(target=self.threadLaunch, args=[commu, passive]).start() for commu in amino_list.comId]
 
-        if "command" in self.commands.keys() or "answer" in self.commands.keys():
+        if self.categorie_exist("command") or self.categorie_exist("answer"):
             self.launch_text_message()
 
-        if "on_member_join_chat" in self.commands.keys():
+        if self.categorie_exist("on_member_join_chat"):
             self.launch_on_member_join_chat()
 
-        if "on_member_leave_chat" in self.commands.keys():
+        if self.categorie_exist("on_member_leave_chat"):
             self.launch_on_member_leave_chat()
 
+        if self.categorie_exist("on_other"):
+            self.launch_other_message()
+
+        if self.categorie_exist("on_remove"):
+            self.launch_removed_message()
+
+        if self.categorie_exist("on_delete"):
+            self.launch_delete_message()
+
+        if self.categorie_exist("on_all"):
+            self.launch_all_message()
+
+    def message_analyse(self, data, type):
+        try:
+            commuId = data.json["ndcId"]
+            subClient = self.get_community(commuId)
+        except Exception:
+            return
+
+        args = Parameters(data, subClient)
+        Thread(target=self.execute, args=[type, args, type]).start()
+
+    def on_member_event(self, data, type):
+        try:
+            commuId = data.json["ndcId"]
+            subClient = self.get_community(commuId)
+        except Exception:
+            return
+
+        args = Parameters(data, subClient)
+
+        if not self.check(args, "bot"):
+            Thread(target=self.execute, args=[type, args, type]).start()
+
     def launch_text_message(self):
-        @self.callbacks.event("on_text_message")
-        def on_text_message(data):
+        def text_message(data):
             try:
                 commuId = data.json["ndcId"]
                 subClient = self.get_community(commuId)
@@ -244,67 +370,119 @@ class BotAmino(Command, Client, TimeOut):
 
             args = Parameters(data, subClient)
 
+            if "on_message" in self.commands.keys():
+                Thread(target=self.execute, args=["on_message", args, "on_message"]).start()
+
+            if not self.check(args, 'staff', 'bot') and subClient.banned_words:
+                self.check_banned_words(args)
+
             if not self.timed_out(args.authorId) and args.message.startswith(subClient.prefix) and not self.check(args, "bot"):
-                subClient.send_message(args.chatId, "You are spamming, be careful")
+                subClient.send_message(args.chatId, self.spam_message)
+                return
 
             elif "command" in self.commands.keys() and args.message.startswith(subClient.prefix) and not self.check(args, "bot"):
                 print(f"{args.author} : {args.message}")
-                command = args.message.split()[0][len(subClient.prefix):]
+                command = args.message.lower().split()[0][len(subClient.prefix):]
                 args.message = ' '.join(args.message.split()[1:])
                 self.time_user(args.authorId, self.wait)
                 if command.lower() in self.commands["command"].keys():
                     Thread(target=self.execute, args=[command, args]).start()
 
+                elif self.no_command_message:
+                    subClient.send_message(args.chatId, self.no_command_message)
+                return
+
             elif "answer" in self.commands.keys() and args.message.lower() in self.commands["answer"] and not self.check(args, "bot"):
                 print(f"{args.author} : {args.message}")
                 self.time_user(args.authorId, self.wait)
                 Thread(target=self.execute, args=[args.message.lower(), args, "answer"]).start()
-            else:
                 return
+        try:
+            @self.callbacks.event("on_text_message")
+            def on_text_message(data):
+                text_message(data)
+        except Exception:
+            @self.event("on_text_message")
+            def on_text_message(data):
+                text_message(data)
+
+    def launch_other_message(self):
+        for type_name in ("on_strike_message", "on_voice_chat_not_answered",
+                          "on_voice_chat_not_cancelled", "on_voice_chat_not_declined",
+                          "on_video_chat_not_answered", "on_video_chat_not_cancelled",
+                          "on_video_chat_not_declined", "on_voice_chat_start", "on_video_chat_start",
+                          "on_voice_chat_end", "on_video_chat_end", "on_screen_room_start",
+                          "on_screen_room_end", "on_avatar_chat_start", "on_avatar_chat_end"):
+            try:
+                @self.callbacks.event(type_name)
+                def on_other_message(data):
+                    self.message_analyse(data, "on_other")
+            except AttributeError:
+                @self.event(type_name)
+                def on_other_message(data):
+                    self.message_analyse(data, "on_other")
+
+    def launch_all_message(self):
+        try:
+            for x in (self.chat_methods):
+                @self.event(self.chat_methods[x].__name__)
+                def on_all_message(data):
+                    self.message_analyse(data, "on_all")
+        except AttributeError:
+            for x in (self.callbacks.chat_methods):
+                @self.event(self.callbacks.chat_methods[x].__name__)
+                def on_all_message(data):
+                    self.message_analyse(data, "on_all")
+
+    def launch_delete_message(self):
+        try:
+            @self.callbacks.event("on_delete_message")
+            def on_delete_message(data):
+                self.message_analyse(data, "on_delete")
+        except AttributeError:
+            @self.event("on_delete_message")
+            def on_delete_message(data):
+                self.message_analyse(data, "on_delete")
+
+    def launch_removed_message(self):
+        for type_name in ("on_chat_removed_message", "on_text_message_force_removed", "on_text_message_removed_by_admin", "on_delete_message"):
+            try:
+                @self.callbacks.event(type_name)
+                def on_chat_removed(data):
+                    self.message_analyse(data, "on_remove")
+            except AttributeError:
+                @self.event(type_name)
+                def on_chat_removed(data):
+                    self.message_analyse(data, "on_remove")
 
     def launch_on_member_join_chat(self):
-        @self.callbacks.event("on_group_member_join")
-        def on_group_member_join(data):
-            try:
-                commuId = data.json["ndcId"]
-                subClient = self.get_community(commuId)
-            except Exception:
-                return
-
-            args = Parameters(data, subClient)
-
-            if not self.check(args, "bot"):
-                if isinstance(self.commands["on_member_join_chat"], dict):
-                    if args.chatId in self.commands["on_member_join_chat"].keys():
-                        Thread(target=self.execute, args=[args.chatId, args, "on_member_join_chat"]).start()
-                else:
-                    Thread(target=self.execute, args=[0, args, "on_member_join_chat"]).start()
+        try:
+            @self.callbacks.event("on_group_member_join")
+            def on_group_member_join(data):
+                self.on_member_event(data, "on_member_join_chat")
+        except AttributeError:
+            @self.event("on_group_member_join")
+            def on_group_member_join(data):
+                self.on_member_event(data, "on_member_join_chat")
 
     def launch_on_member_leave_chat(self):
-        @self.callbacks.event("on_group_member_leave")
-        def on_group_member_leave(data):
-            try:
-                commuId = data.json["ndcId"]
-                subClient = self.get_community(commuId)
-            except Exception:
-                return
-
-            args = Parameters(data, subClient)
-
-            if not self.check(args, "bot"):
-                if isinstance(self.commands["on_member_leave_chat"], dict):
-                    if args.chatId in self.commands["on_member_leave_chat"].keys():
-                        Thread(target=self.execute, args=[args.chatId, args, "on_member_leave_chat"]).start()
-                else:
-                    Thread(target=self.execute, args=[0, args, "on_member_leave_chat"]).start()
+        try:
+            @self.callbacks.event("on_group_member_leave")
+            def on_group_member_leave(data):
+                self.on_member_event(data, "on_member_leave_chat")
+        except AttributeError:
+            @self.event("on_group_member_leave")
+            def on_group_member_leave(data):
+                self.on_member_event(data, "on_member_leave_chat")
 
 
 class Bot(SubClient, ACM):
-    def __init__(self, client, community, prefix: str = "!", bio=None):
+    def __init__(self, client, community, prefix: str = "!", bio=None, activity=False) -> None:
         self.client = client
         self.marche = True
         self.prefix = prefix
         self.bio_contents = bio
+        self.activity = activity
 
         if isinstance(community, int):
             self.community_id = community
@@ -341,7 +519,11 @@ class Bot(SubClient, ACM):
         old_dict = self.get_file_dict()
         new_dict = self.create_dict()
 
-        {**new_dict, **{i: e for i, e in old_dict.items() if i in new_dict}}
+        def do(k, v): old_dict[k] = v
+        def undo(k): del old_dict[k]
+
+        [do(k, v) for k, v in new_dict.items() if k not in old_dict]
+        [undo(k) for k in new_dict.keys() if k not in old_dict]
 
         self.update_file(old_dict)
 
@@ -349,20 +531,18 @@ class Bot(SubClient, ACM):
 
         self.banned_words = self.get_file_info("banned_words")
         self.message_bvn = self.get_file_info("welcome")
-        self.locked_command = self.get_file_info("locked_command")
-        self.admin_locked_command = self.get_file_info("admin_locked_command")
         self.welcome_chat = self.get_file_info("welcome_chat")
         self.prefix = self.get_file_info("prefix")
-        self.level = self.get_file_info("level")
         self.favorite_users = self.get_file_info("favorite_users")
         self.favorite_chats = self.get_file_info("favorite_chats")
+        self.update_file()
         self.activity_status("on")
         new_users = self.get_all_users(start=0, size=30, type="recent")
 
         self.new_users = [elem["uid"] for elem in new_users.json["userProfileList"]]
+
         if self.welcome_chat or self.message_bvn:
-            with suppress(Exception):
-                Thread(target=self.check_new_member).start()
+            Thread(target=self.check_new_member).start()
 
     def create_community_file(self):
         with open(f'{path_amino}/{self.community_amino_id}.json', 'w', encoding='utf8') as file:
@@ -370,10 +550,10 @@ class Bot(SubClient, ACM):
             file.write(dumps(dict, sort_keys=False, indent=4))
 
     def create_dict(self):
-        return {"welcome": "", "banned_words": [], "locked_command": [], "admin_locked_command": [], "prefix": self.prefix, "welcome_chat": "", "level": 0, "favorite_users": [], "favorite_chats": []}
+        return {"welcome": "", "prefix": self.prefix, "welcome_chat": "", "favorite_users": [], "favorite_chats": [], "banned_words": []}
 
     def get_dict(self):
-        return {"welcome": self.message_bvn, "banned_words": self.banned_words, "locked_command": self.locked_command, "admin_locked_command": self.admin_locked_command, "prefix": self.prefix, "welcome_chat": self.welcome_chat, "level": self.level, "favorite_users": self.favorite_users, "favorite_chats": self.favorite_chats}
+        return {"welcome": self.message_bvn, "prefix": self.prefix, "welcome_chat": self.welcome_chat, "favorite_users": self.favorite_users, "favorite_chats": self.favorite_chats, "banned_words": self.banned_words}
 
     def update_file(self, dict=None):
         if not dict:
@@ -389,14 +569,13 @@ class Bot(SubClient, ACM):
         with open(f"{path_amino}/{self.community_amino_id}.json", "r", encoding="utf8") as file:
             return load(file)
 
+    def get_banned_words(self):
+        return self.banned_words
+
     def set_prefix(self, prefix: str):
         self.prefix = prefix
         self.update_file()
 
-    def set_level(self, level: int):
-        self.level = level
-        self.update_file()
-
     def set_welcome_message(self, message: str):
         self.message_bvn = message.replace('"', '“')
         self.update_file()
@@ -405,18 +584,6 @@ class Bot(SubClient, ACM):
         self.welcome_chat = chatId
         self.update_file()
 
-    def add_locked_command(self, liste: list):
-        self.locked_command.extend(liste)
-        self.update_file()
-
-    def add_admin_locked_command(self, liste: list):
-        self.admin_locked_command.extend(liste)
-        self.update_file()
-
-    def add_banned_words(self, liste: list):
-        self.banned_words.extend(liste)
-        self.update_file()
-
     def add_favorite_users(self, value: str):
         self.favorite_users.append(value)
         self.update_file()
@@ -425,16 +592,8 @@ class Bot(SubClient, ACM):
         self.favorite_chats.append(value)
         self.update_file()
 
-    def remove_locked_command(self, liste: list):
-        [self.locked_command.remove(elem) for elem in liste if elem in self.locked_command]
-        self.update_file()
-
-    def remove_admin_locked_command(self, liste: list):
-        [self.admin_locked_command.remove(elem) for elem in liste if elem in self.admin_locked_command]
-        self.update_file()
-
-    def remove_banned_words(self, liste: list):
-        [self.banned_words.remove(elem) for elem in liste if elem in self.banned_words]
+    def add_banned_words(self, liste: list):
+        self.banned_words.extend(liste)
         self.update_file()
 
     def remove_favorite_users(self, value: str):
@@ -447,6 +606,10 @@ class Bot(SubClient, ACM):
         [self.favorite_chats.remove(elem) for elem in liste if elem in self.favorite_chats]
         self.update_file()
 
+    def remove_banned_words(self, liste: list):
+        [self.banned_words.remove(elem) for elem in liste if elem in self.banned_words]
+        self.update_file()
+
     def unset_welcome_chat(self):
         self.welcome_chat = ""
         self.update_file()
@@ -513,6 +676,13 @@ class Bot(SubClient, ACM):
         return lower_name if lower_name else None
 
     def ask_all_members(self, message, lvl: int = 20, type_bool: int = 1):
+        def ask(uid):
+            print(uid)
+            try:
+                self.start_chat(userId=[uid], message=message)
+            except Exception:
+                self.start_chat(userId=[uid], message=message)
+
         size = self.get_all_users(start=0, size=1, type="recent").json['userProfileCount']
         st = 0
 
@@ -522,12 +692,11 @@ class Bot(SubClient, ACM):
                 value = 100
             users = self.get_all_users(start=st, size=value)
             if type_bool == 1:
-                user_lvl_list = [user['uid'] for user in users.json['userProfileList'] if user['level'] == lvl]
+                [ask(user["uid"]) for user in users.json['userProfileList'] if user['level'] == lvl]
             elif type_bool == 2:
-                user_lvl_list = [user['uid'] for user in users.json['userProfileList'] if user['level'] <= lvl]
+                [ask(user["uid"]) for user in users.json['userProfileList'] if user['level'] <= lvl]
             elif type_bool == 3:
-                user_lvl_list = [user['uid'] for user in users.json['userProfileList'] if user['level'] >= lvl]
-            self.start_chat(userId=user_lvl_list, message=message)
+                [ask(user["uid"]) for user in users.json['userProfileList'] if user['level'] >= lvl]
             size -= 100
             st += 100
 
@@ -551,6 +720,10 @@ class Bot(SubClient, ACM):
     def stop_instance(self):
         self.marche = False
 
+    def start_instance(self):
+        self.marche = True
+        Thread(target=self.passive).start()
+
     def leave_community(self):
         self.client.leave_community(comId=self.community_id)
         self.marche = False
@@ -559,20 +732,19 @@ class Bot(SubClient, ACM):
                 self.leave_chat(elem)
 
     def check_new_member(self):
-        if not (self.message_bvn and self.welcome_chat):
+        if not (self.message_bvn or self.welcome_chat):
             return
         new_list = self.get_all_users(start=0, size=25, type="recent")
         new_member = [(elem["nickname"], elem["uid"]) for elem in new_list.json["userProfileList"]]
         for elem in new_member:
             name, uid = elem[0], elem[1]
-            try:
-                val = self.get_wall_comments(userId=uid, sorting='newest').commentId
-            except Exception:
-                val = True
+
+            val = self.get_wall_comments(userId=uid, sorting='newest').commentId
 
             if not val and self.message_bvn:
                 with suppress(Exception):
                     self.comment(message=self.message_bvn, userId=uid)
+
             if not val and self.welcome_chat:
                 with suppress(Exception):
                     self.send_message(chatId=self.welcome_chat, message=f"Welcome here ‎‏‎‏@{name}!‬‭", mentionUserIds=[uid])
@@ -587,10 +759,7 @@ class Bot(SubClient, ACM):
         for elem in new_member:
             name, uid = elem[0], elem[1]
 
-            try:
-                val = self.get_wall_comments(userId=uid, sorting='newest').commentId
-            except Exception:
-                val = True
+            val = self.get_wall_comments(userId=uid, sorting='newest').commentId
 
             if not val and uid not in self.new_users and self.message_bvn:
                 with suppress(Exception):
@@ -618,22 +787,16 @@ class Bot(SubClient, ACM):
     def get_member_level(self, uid):
         return self.get_user_info(userId=uid).level
 
-    def is_level_good(self, uid):
-        return self.get_user_info(userId=uid).level >= self.level
-
     def get_member_titles(self, uid):
         with suppress(Exception):
             return self.get_user_info(userId=uid).customTitles
         return False
 
-    def get_member_info(self, uid):
-        return self.get_user_info(userId=uid)
-
     def get_wallet_amount(self):
         return self.client.get_wallet_info().totalCoins
 
     def generate_transaction_id(self):
-        return f"{''.join(sample([lst for lst in hexdigits[:-6]], 8))}-{''.join(sample([lst for lst in hexdigits[:-6]], 4))}-{''.join(sample([lst for lst in hexdigits[:-6]], 4))}-{''.join(sample([lst for lst in hexdigits[:-6]], 4))}-{''.join(sample([lst for lst in hexdigits[:-6]], 12))}"
+        return str(uuid4())
 
     def pay(self, coins: int = 0, blogId: str = None, chatId: str = None, objectId: str = None, transactionId: str = None):
         if not transactionId:
@@ -690,27 +853,28 @@ class Bot(SubClient, ACM):
     def unfollow_user(self, uid):
         self.unfollow(userId=uid)
 
-    def add_title(self, uid, title: str, color: str = None):
+    def add_title(self, uid: str, title: str, color: str = None):
         member = self.get_member_titles(uid)
-        titles = [i['title'] for i in member] + [title]
-        colors = [i['color'] for i in member] + [color]
+        try:
+            titles = [i['title'] for i in member] + [title]
+            colors = [i['color'] for i in member] + [color]
+        except TypeError:
+            titles = [title]
+            colors = [color]
 
         self.edit_titles(uid, titles, colors)
         return True
 
-    def remove_title(self, uid, title: str):
+    def remove_title(self, uid: str, title: str):
         member = self.get_member_titles(uid)
         tlist = []
         clist = []
-        for elem in member:
-            tlist.append(elem["title"])
-            clist.append(elem["color"])
 
-        if title in tlist:
-            nb = tlist.index(title)
-            tlist.pop(nb)
-            clist.pop(nb)
-            self.edit_titles(uid, tlist, clist)
+        for t in member:
+            if t["title"] != title:
+                tlist.append(t["title"])
+                clist.append(t["color"])
+        self.edit_titles(uid, tlist, clist)
         return True
 
     def passive(self):
@@ -740,18 +904,19 @@ class Bot(SubClient, ACM):
             except Exception as e:
                 print_exception(e)
 
-        slp(30)
-        change_bio_and_welcome_members()
         feature_chats()
         feature_users()
 
-        every().minute.do(change_bio_and_welcome_members)
-        every(2).hours.do(feature_chats)
-        every().day.do(feature_users)
-
+        j = 0
+        k = 0
         while self.marche:
-            run_pending()
-            slp(10)
-
-    def run(self):
-        Thread(target=self.passive).start()
+            change_bio_and_welcome_members()
+            if j >= 120:
+                feature_chats()
+                j = 0
+            if k >= 1440:
+                feature_users()
+                k = 0
+            slp(60)
+            j += 1
+            k += 1
